@@ -1,38 +1,68 @@
-/**********************************************************************
- *
- * Copyright (C) Imagination Technologies Ltd. All rights reserved.
- * 
- * This program is free software; you can redistribute it and/or modify it
- * under the terms and conditions of the GNU General Public License,
- * version 2, as published by the Free Software Foundation.
- * 
- * This program is distributed in the hope it will be useful but, except 
- * as otherwise stated in writing, without any warranty; without even the 
- * implied warranty of merchantability or fitness for a particular purpose. 
- * See the GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
- * 
- * The full GNU General Public License is included in this distribution in
- * the file called "COPYING".
- *
- * Contact Information:
- * Imagination Technologies Ltd. <gpl-support@imgtec.com>
- * Home Park Estate, Kings Langley, Herts, WD4 8LZ, UK 
- *
- ******************************************************************************/
+/*************************************************************************/ /*!
+@Title          Device specific reset routines
+@Copyright      Copyright (c) Imagination Technologies Ltd. All Rights Reserved
+@License        Dual MIT/GPLv2
+
+The contents of this file are subject to the MIT license as set out below.
+
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+Alternatively, the contents of this file may be used under the terms of
+the GNU General Public License Version 2 ("GPL") in which case the provisions
+of GPL are applicable instead of those above.
+
+If you wish to allow use of your version of this file only under the terms of
+GPL, and not to allow others to use your version of this file under the terms
+of the MIT license, indicate your decision by deleting the provisions above
+and replace them with the notice and other provisions required by GPL as set
+out in the file called "GPL-COPYING" included in this distribution. If you do
+not delete the provisions above, a recipient may use your version of this file
+under the terms of either the MIT license or GPL.
+
+This License is also included in this distribution in the file called
+"MIT-COPYING".
+
+EXCEPT AS OTHERWISE STATED IN A NEGOTIATED AGREEMENT: (A) THE SOFTWARE IS
+PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING
+BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
+PURPOSE AND NONINFRINGEMENT; AND (B) IN NO EVENT SHALL THE AUTHORS OR
+COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
+IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+*/ /**************************************************************************/
 
 #include "sgxdefs.h"
 #include "sgxmmu.h"
 #include "services_headers.h"
 #include "sgxinfokm.h"
 #include "sgxconfig.h"
+#include "sgxutils.h"
 
 #include "pdump_km.h"
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXInitClocks
+
+ @Description
+	Initialise the SGX clocks
+
+ @Input psDevInfo - device info. structure
+ @Input ui32PDUMPFlags - flags to control PDUMP output
+
+ @Return   IMG_VOID
+
+******************************************************************************/
 IMG_VOID SGXInitClocks(PVRSRV_SGXDEV_INFO	*psDevInfo,
 					   IMG_UINT32			ui32PDUMPFlags)
 {
@@ -40,7 +70,7 @@ IMG_VOID SGXInitClocks(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 	
 	ui32RegVal = psDevInfo->ui32ClkGateCtl;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_CLKGATECTL, ui32RegVal);
@@ -54,6 +84,19 @@ IMG_VOID SGXInitClocks(PVRSRV_SGXDEV_INFO	*psDevInfo,
 }
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXResetInitBIFContexts
+
+ @Description
+	Initialise the BIF memory contexts
+
+ @Input psDevInfo - SGX Device Info
+
+ @Return   IMG_VOID
+
+******************************************************************************/
 static IMG_VOID SGXResetInitBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 										IMG_UINT32			ui32PDUMPFlags)
 {
@@ -61,7 +104,7 @@ static IMG_VOID SGXResetInitBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 
 	ui32RegVal = 0;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_CTRL, ui32RegVal);
@@ -73,7 +116,7 @@ static IMG_VOID SGXResetInitBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_BANK_SET, ui32RegVal, ui32PDUMPFlags);
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_BANK0, ui32RegVal);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_BANK0, ui32RegVal, ui32PDUMPFlags);
-#endif 
+#endif /* SGX_FEATURE_MULTIPLE_MEM_CONTEXTS */
 
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Initialise the BIF directory list\r\n");
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_DIR_LIST_BASE0, ui32RegVal);
@@ -92,10 +135,23 @@ static IMG_VOID SGXResetInitBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 			PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, ui32DirListReg, ui32RegVal, ui32PDUMPFlags);
 		}
 	}
-#endif 
+#endif /* SGX_FEATURE_MULTIPLE_MEM_CONTEXTS */
 }
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXResetSetupBIFContexts
+
+ @Description
+	Configure the BIF for the EDM context
+
+ @Input psDevInfo - SGX Device Info
+
+ @Return   IMG_VOID
+
+******************************************************************************/
 static IMG_VOID SGXResetSetupBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 										 IMG_UINT32			ui32PDUMPFlags)
 {
@@ -103,37 +159,37 @@ static IMG_VOID SGXResetSetupBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 	
 	#if defined(SGX_FEATURE_MULTIPLE_MEM_CONTEXTS)
-	
+	/* Set up EDM for bank 0 to point at kernel context */
 	ui32RegVal = (SGX_BIF_DIR_LIST_INDEX_EDM << EUR_CR_BIF_BANK0_INDEX_EDM_SHIFT);
 
 	#if defined(SGX_FEATURE_2D_HARDWARE) && !defined(SGX_FEATURE_PTLA)
-	
+	/* Set up 2D core for bank 0 to point at kernel context */
 	ui32RegVal |= (SGX_BIF_DIR_LIST_INDEX_EDM << EUR_CR_BIF_BANK0_INDEX_2D_SHIFT);
-	#endif 
+	#endif /* SGX_FEATURE_2D_HARDWARE */
 
 	#if defined(FIX_HW_BRN_23410)
-	
+	/* Set up TA core for bank 0 to point at kernel context to guarantee it is a valid context */
 	ui32RegVal |= (SGX_BIF_DIR_LIST_INDEX_EDM << EUR_CR_BIF_BANK0_INDEX_TA_SHIFT);
-	#endif 
+	#endif /* FIX_HW_BRN_23410 */
 
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_BANK0, ui32RegVal);
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Set up EDM requestor page table in BIF\r\n");
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_BANK0, ui32RegVal, ui32PDUMPFlags);
-	#endif 
+	#endif /* defined(SGX_FEATURE_MULTIPLE_MEM_CONTEXTS) */
 
 	{
 		IMG_UINT32	ui32EDMDirListReg;
 
-		
+		/* Set up EDM context with kernel page directory */
 		#if (SGX_BIF_DIR_LIST_INDEX_EDM == 0)
 		ui32EDMDirListReg = EUR_CR_BIF_DIR_LIST_BASE0;
 		#else
-		
+		/* Bases 0 and 1 are not necessarily contiguous */
 		ui32EDMDirListReg = EUR_CR_BIF_DIR_LIST_BASE1 + 4 * (SGX_BIF_DIR_LIST_INDEX_EDM - 1);
-		#endif 
+		#endif /* SGX_BIF_DIR_LIST_INDEX_EDM */
 
 		ui32RegVal = psDevInfo->sKernelPDDevPAddr.uiAddr >> SGX_MMU_PDE_ADDR_ALIGNSHIFT;
 		
@@ -149,6 +205,23 @@ static IMG_VOID SGXResetSetupBIFContexts(PVRSRV_SGXDEV_INFO	*psDevInfo,
 }
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXResetSleep
+
+ @Description
+
+ Sleep for a short time to allow reset register writes to complete.
+ Required because no status registers are available to poll on.
+
+ @Input psDevInfo - SGX Device Info
+ @Input ui32PDUMPFlags - flags to control PDUMP output
+ @Input bPDump - Pdump the sleep
+
+ @Return   Nothing
+
+******************************************************************************/
 static IMG_VOID SGXResetSleep(PVRSRV_SGXDEV_INFO	*psDevInfo,
 							  IMG_UINT32			ui32PDUMPFlags,
 							  IMG_BOOL				bPDump)
@@ -160,15 +233,15 @@ static IMG_VOID SGXResetSleep(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	ui32ReadRegister = EUR_CR_MASTER_SOFT_RESET;
 	#else
 	ui32ReadRegister = EUR_CR_SOFT_RESET;
-	#endif 
+	#endif /* SGX_FEATURE_MP */
 #endif
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 
-	
-	OSWaitus(100 * 1000000 / psDevInfo->ui32CoreClockSpeed);
+	/* Sleep for 100 SGX clocks */
+	SGXWaitClocks(psDevInfo, 100);
 	if (bPDump)
 	{
 		PDUMPIDLWITHFLAGS(30, ui32PDUMPFlags);
@@ -179,14 +252,32 @@ static IMG_VOID SGXResetSleep(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	}
 
 #if defined(EMULATOR)
-	
-
+	/*
+		Read a register to make sure we wait long enough on the emulator...
+	*/
 	OSReadHWReg(psDevInfo->pvRegsBaseKM, ui32ReadRegister);
 #endif
 }
 
 
 #if !defined(SGX_FEATURE_MP)
+/*!
+*******************************************************************************
+
+ @Function	SGXResetSoftReset
+
+ @Description
+
+ Write to the SGX soft reset register.
+
+ @Input psDevInfo - SGX Device Info
+ @Input bResetBIF - Include the BIF in the soft reset
+ @Input ui32PDUMPFlags - flags to control PDUMP output
+ @Input bPDump - Pdump the sleep
+
+ @Return   Nothing
+
+******************************************************************************/
 static IMG_VOID SGXResetSoftReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 								  IMG_BOOL				bResetBIF,
 								  IMG_UINT32			ui32PDUMPFlags,
@@ -195,13 +286,14 @@ static IMG_VOID SGXResetSoftReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	IMG_UINT32 ui32SoftResetRegVal;
 
 	ui32SoftResetRegVal =
-					
+					/* add common reset bits: */
 					EUR_CR_SOFT_RESET_DPM_RESET_MASK |
 					EUR_CR_SOFT_RESET_TA_RESET_MASK  |
 					EUR_CR_SOFT_RESET_USE_RESET_MASK |
 					EUR_CR_SOFT_RESET_ISP_RESET_MASK |
 					EUR_CR_SOFT_RESET_TSP_RESET_MASK;
 
+/* add conditional reset bits: */
 #ifdef EUR_CR_SOFT_RESET_TWOD_RESET_MASK
 	ui32SoftResetRegVal |= EUR_CR_SOFT_RESET_TWOD_RESET_MASK;
 #endif
@@ -253,7 +345,7 @@ static IMG_VOID SGXResetSoftReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 
 	if (bResetBIF)
 	{
@@ -268,13 +360,28 @@ static IMG_VOID SGXResetSoftReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 }
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXResetInvalDC
+
+ @Description
+
+ Invalidate the BIF Directory Cache and wait for the operation to complete.
+
+ @Input psDevInfo - SGX Device Info
+ @Input ui32PDUMPFlags - flags to control PDUMP output
+
+ @Return   Nothing
+
+******************************************************************************/
 static IMG_VOID SGXResetInvalDC(PVRSRV_SGXDEV_INFO	*psDevInfo,
 							    IMG_UINT32			ui32PDUMPFlags,
 								IMG_BOOL			bPDump)
 {
 	IMG_UINT32 ui32RegVal;
 
-	
+	/* Invalidate BIF Directory cache. */
 #if defined(EUR_CR_BIF_CTRL_INVAL)
 	ui32RegVal = EUR_CR_BIF_CTRL_INVAL_ALL_MASK;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_CTRL_INVAL, ui32RegVal);
@@ -301,9 +408,10 @@ static IMG_VOID SGXResetInvalDC(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(SGX_FEATURE_MULTIPLE_MEM_CONTEXTS)
 	{
-		
-
-
+		/*
+			Wait for the DC invalidate to complete - indicated by
+			outstanding reads reaching zero.
+		*/
 		if (PollForValueKM((IMG_UINT32 *)((IMG_UINT8*)psDevInfo->pvRegsBaseKM + EUR_CR_BIF_MEM_REQ_STAT),
 							0,
 							EUR_CR_BIF_MEM_REQ_STAT_READS_MASK,
@@ -320,11 +428,28 @@ static IMG_VOID SGXResetInvalDC(PVRSRV_SGXDEV_INFO	*psDevInfo,
 			PDUMPREGPOLWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_MEM_REQ_STAT, 0, EUR_CR_BIF_MEM_REQ_STAT_READS_MASK, ui32PDUMPFlags, PDUMP_POLL_OPERATOR_EQUAL);
 		}
 	}
-#endif 
+#endif /* SGX_FEATURE_MULTIPLE_MEM_CONTEXTS */
 }
-#endif 
+#endif /* SGX_FEATURE_MP */
 
 
+/*!
+*******************************************************************************
+
+ @Function	SGXReset
+
+ @Description
+
+ Reset chip
+
+ @Input psDevInfo - device info. structure
+ @Input bHardwareRecovery - true if recovering powered hardware,
+ 							false if powering up
+ @Input ui32PDUMPFlags - flags to control PDUMP output
+
+ @Return   IMG_VOID
+
+******************************************************************************/
 IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 				  IMG_BOOL				bHardwareRecovery,
 				  IMG_UINT32			ui32PDUMPFlags)
@@ -339,12 +464,12 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Start of SGX reset sequence\r\n");
 
 #if defined(FIX_HW_BRN_23944)
-	
+	/* Pause the BIF. */
 	ui32RegVal = EUR_CR_BIF_CTRL_PAUSE_MASK;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_CTRL, ui32RegVal);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_CTRL, ui32RegVal, ui32PDUMPFlags);
@@ -354,7 +479,7 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	ui32RegVal = OSReadHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_INT_STAT);
 	if (ui32RegVal & ui32BifFaultMask)
 	{
-		
+		/* Page fault needs to be cleared before resetting the BIF. */
 		ui32RegVal = EUR_CR_BIF_CTRL_PAUSE_MASK | EUR_CR_BIF_CTRL_CLEAR_FAULT_MASK;
 		OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_CTRL, ui32RegVal);
 		PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_CTRL, ui32RegVal, ui32PDUMPFlags);
@@ -367,17 +492,18 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 		SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_TRUE);
 	}
-#endif 
+#endif /* defined(FIX_HW_BRN_23944) */
 
-	
+	/* Reset all including BIF */
 	SGXResetSoftReset(psDevInfo, IMG_TRUE, ui32PDUMPFlags, IMG_TRUE);
 
 	SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_TRUE);
 
-	
-
+	/*
+		Initialise the BIF state.
+	*/
 #if defined(SGX_FEATURE_36BIT_MMU)
-	
+	/* enable 36bit addressing mode if the MMU supports it*/
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_36BIT_ADDRESSING, EUR_CR_BIF_36BIT_ADDRESSING_ENABLE_MASK);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_36BIT_ADDRESSING, EUR_CR_BIF_36BIT_ADDRESSING_ENABLE_MASK, ui32PDUMPFlags);
 #endif
@@ -385,57 +511,60 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	SGXResetInitBIFContexts(psDevInfo, ui32PDUMPFlags);
 
 #if defined(EUR_CR_BIF_MEM_ARB_CONFIG)
-	
-
+	/*
+		Initialise the memory arbiter to its default state
+	*/
 	ui32RegVal	= (12UL << EUR_CR_BIF_MEM_ARB_CONFIG_PAGE_SIZE_SHIFT) |
 				  (7UL << EUR_CR_BIF_MEM_ARB_CONFIG_BEST_CNT_SHIFT) |
 				  (12UL << EUR_CR_BIF_MEM_ARB_CONFIG_TTE_THRESH_SHIFT);
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_MEM_ARB_CONFIG, ui32RegVal);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_MEM_ARB_CONFIG, ui32RegVal, ui32PDUMPFlags);
-#endif 
+#endif /* EUR_CR_BIF_MEM_ARB_CONFIG */
 
 #if defined(SGX_FEATURE_SYSTEM_CACHE)
 	#if defined(SGX_BYPASS_SYSTEM_CACHE)
-		
+		/* set the SLC to bypass all accesses */
 		ui32RegVal = MNE_CR_CTRL_BYPASS_ALL_MASK;
 	#else
 		#if defined(FIX_HW_BRN_26620)
 			ui32RegVal = 0;
 		#else
-			
+			/* set the SLC to bypass cache-coherent accesses */
 			ui32RegVal = MNE_CR_CTRL_BYP_CC_MASK;
 		#endif
 		#if defined(FIX_HW_BRN_34028)
-			
+			/* Bypass the MNE for the USEC requester */
 			ui32RegVal |= (8 << MNE_CR_CTRL_BYPASS_SHIFT);
 		#endif
-	#endif 
+	#endif /* SGX_BYPASS_SYSTEM_CACHE */
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, MNE_CR_CTRL, ui32RegVal);
 	PDUMPREG(SGX_PDUMPREG_NAME, MNE_CR_CTRL, ui32RegVal);
-#endif 
+#endif /* SGX_FEATURE_SYSTEM_CACHE */
 
 	if (bHardwareRecovery)
 	{
-		
-
-
-
-
-
-
+		/*
+			Set all requestors to the dummy PD which forces all memory
+			accesses to page fault.
+			This enables us to flush out BIF requests from parts of SGX
+			which do not have their own soft reset.
+			Note: sBIFResetPDDevPAddr.uiAddr is a relative address (2GB max)
+			MSB is the bus master flag; 1 == enabled
+		*/
 		ui32RegVal = (IMG_UINT32)psDevInfo->sBIFResetPDDevPAddr.uiAddr;
 		OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_DIR_LIST_BASE0, ui32RegVal);
 
 		SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
-		
+		/* Bring BIF out of reset. */
 		SGXResetSoftReset(psDevInfo, IMG_FALSE, ui32PDUMPFlags, IMG_TRUE);
 		SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
 		SGXResetInvalDC(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
-		
-
+		/*
+			Check for a page fault from parts of SGX which do not have a reset.
+		*/
 		for (;;)
 		{
 			IMG_UINT32 ui32BifIntStat = OSReadHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_INT_STAT);
@@ -447,18 +576,19 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 				break;
 			}
 
-			
-
-
+			/*
+				There is a page fault, so reset the BIF again, map in the dummy page,
+				bring the BIF up and invalidate the Directory Cache.
+			*/
 			sBifFault.uiAddr = OSReadHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_FAULT);
 			PVR_DPF((PVR_DBG_WARNING, "SGXReset: Page fault 0x%x/0x%x", ui32BifIntStat, sBifFault.uiAddr));
 			ui32PDIndex = sBifFault.uiAddr >> (SGX_MMU_PAGE_SHIFT + SGX_MMU_PT_SHIFT);
 			ui32PTIndex = (sBifFault.uiAddr & SGX_MMU_PT_MASK) >> SGX_MMU_PAGE_SHIFT;
 
-			
+			/* Put the BIF into reset. */
 			SGXResetSoftReset(psDevInfo, IMG_TRUE, ui32PDUMPFlags, IMG_FALSE);
 
-			
+			/* Map in the dummy page. */
 			psDevInfo->pui32BIFResetPD[ui32PDIndex] = (psDevInfo->sBIFResetPTDevPAddr.uiAddr
 													>>SGX_MMU_PDE_ADDR_ALIGNSHIFT)
 													| SGX_MMU_PDE_PAGE_SIZE_4K
@@ -467,7 +597,7 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 													>>SGX_MMU_PTE_ADDR_ALIGNSHIFT)
 													| SGX_MMU_PTE_VALID;
 
-			
+			/* Clear outstanding events. */
 			ui32RegVal = OSReadHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_EVENT_STATUS);
 			OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_EVENT_HOST_CLEAR, ui32RegVal);
 			ui32RegVal = OSReadHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_EVENT_STATUS2);
@@ -475,50 +605,51 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 			SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
-			
+			/* Bring the BIF out of reset. */
 			SGXResetSoftReset(psDevInfo, IMG_FALSE, ui32PDUMPFlags, IMG_FALSE);
 			SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
-			
+			/* Invalidate Directory Cache. */
 			SGXResetInvalDC(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 
-			
+			/* Unmap the dummy page and try again. */
 			psDevInfo->pui32BIFResetPD[ui32PDIndex] = 0;
 			psDevInfo->pui32BIFResetPT[ui32PTIndex] = 0;
 		}
 	}
 	else
 	{
-		
+		/* Bring BIF out of reset. */
 		SGXResetSoftReset(psDevInfo, IMG_FALSE, ui32PDUMPFlags, IMG_TRUE);
 		SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_FALSE);
 	}	
 
-	
-
+	/*
+		Initialise the BIF memory contexts before bringing the rest of SGX out of reset.
+	*/
 	SGXResetSetupBIFContexts(psDevInfo, ui32PDUMPFlags);
 
 #if defined(SGX_FEATURE_2D_HARDWARE) && !defined(SGX_FEATURE_PTLA)
-	
+	/* check that the heap base has the right alignment (1Mb) */
 	#if ((SGX_2D_HEAP_BASE & ~EUR_CR_BIF_TWOD_REQ_BASE_ADDR_MASK) != 0)
 		#error "SGXReset: SGX_2D_HEAP_BASE doesn't match EUR_CR_BIF_TWOD_REQ_BASE_ADDR_MASK alignment"
 	#endif
-	
+	/* Set up 2D requestor base */
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_BIF_TWOD_REQ_BASE, SGX_2D_HEAP_BASE);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_BIF_TWOD_REQ_BASE, SGX_2D_HEAP_BASE, ui32PDUMPFlags);
 #endif
 
-	
+	/* Invalidate BIF Directory cache. */
 	SGXResetInvalDC(psDevInfo, ui32PDUMPFlags, IMG_TRUE);
 
 	PVR_DPF((PVR_DBG_MESSAGE,"Soft Reset of SGX"));
 
-	
+	/* Take chip out of reset */
 	ui32RegVal = 0;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_SOFT_RESET, ui32RegVal);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_SOFT_RESET, ui32RegVal, ui32PDUMPFlags);
 
-	
+	/* wait a bit */
 	SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_TRUE);
 
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "End of SGX reset sequence\r\n");
@@ -533,16 +664,21 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if !defined(PDUMP)
 	PVR_UNREFERENCED_PARAMETER(ui32PDUMPFlags);
-#endif 
+#endif /* PDUMP */
 
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Start of SGX MP reset sequence\r\n");
 
-	
+	/* Put hydra into soft reset */
 	ui32RegVal = EUR_CR_MASTER_SOFT_RESET_BIF_RESET_MASK  |
 				 EUR_CR_MASTER_SOFT_RESET_IPF_RESET_MASK  |
 				 EUR_CR_MASTER_SOFT_RESET_DPM_RESET_MASK  |
 				 EUR_CR_MASTER_SOFT_RESET_VDM_RESET_MASK;
 
+	if (bHardwareRecovery)
+	{
+		ui32RegVal |= EUR_CR_MASTER_SOFT_RESET_MCI_RESET_MASK;
+	}
+
 #if defined(SGX_FEATURE_PTLA)
 	ui32RegVal |= EUR_CR_MASTER_SOFT_RESET_PTLA_RESET_MASK;
 #endif
@@ -550,7 +686,7 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	ui32RegVal |= EUR_CR_MASTER_SOFT_RESET_SLC_RESET_MASK;
 #endif
 
-	
+	/* Hard reset the slave cores */
 	ui32RegVal |= EUR_CR_MASTER_SOFT_RESET_CORE_RESET_MASK(0)  |
 				  EUR_CR_MASTER_SOFT_RESET_CORE_RESET_MASK(1)  |
 				  EUR_CR_MASTER_SOFT_RESET_CORE_RESET_MASK(2)  |
@@ -569,7 +705,7 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if defined(SGX_FEATURE_SYSTEM_CACHE)
 	#if defined(SGX_BYPASS_SYSTEM_CACHE)
-		#error SGX_BYPASS_SYSTEM_CACHE not supported
+		ui32RegVal = EUR_CR_MASTER_SLC_CTRL_BYPASS_ALL_MASK;
 	#else
 		ui32RegVal = EUR_CR_MASTER_SLC_CTRL_USSE_INVAL_REQ0_MASK |
 		#if defined(FIX_HW_BRN_30954)
@@ -577,6 +713,9 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 		#endif
 		#if defined(PVR_SLC_8KB_ADDRESS_MODE)
 						(4 << EUR_CR_MASTER_SLC_CTRL_ADDR_DECODE_MODE_SHIFT) |
+		#endif
+		#if defined(FIX_HW_BRN_33809)
+			(2 << EUR_CR_MASTER_SLC_CTRL_ADDR_DECODE_MODE_SHIFT) |
 		#endif
 						(0xC << EUR_CR_MASTER_SLC_CTRL_ARB_PAGE_SIZE_SHIFT);
 		OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_MASTER_SLC_CTRL, ui32RegVal);
@@ -594,15 +733,15 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 				EUR_CR_MASTER_SLC_CTRL_BYPASS_REQ_USE3_MASK |
 				EUR_CR_MASTER_SLC_CTRL_BYPASS_REQ_TA_MASK;
 	#endif
+	#endif /* SGX_BYPASS_SYSTEM_CACHE */
 		OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_MASTER_SLC_CTRL_BYPASS, ui32RegVal);
 		PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Initialise the hydra SLC bypass control\r\n");
 		PDUMPREG(SGX_PDUMPREG_NAME, EUR_CR_MASTER_SLC_CTRL_BYPASS, ui32RegVal);
-	#endif 
-#endif 
+#endif /* SGX_FEATURE_SYSTEM_CACHE */
 
 	SGXResetSleep(psDevInfo, ui32PDUMPFlags, IMG_TRUE);
 
-	
+	/* Remove the resets */
 	ui32RegVal = 0;
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_MASTER_SOFT_RESET, ui32RegVal);
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "Remove the resets from all of SGX\r\n");
@@ -619,32 +758,32 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 
 #if defined(FIX_HW_BRN_31278) || defined(FIX_HW_BRN_31620) || defined(FIX_HW_BRN_31671) || defined(FIX_HW_BRN_32085)
 	#if defined(FIX_HW_BRN_31278) || defined(FIX_HW_BRN_32085)
-	
+	/* disable prefetch */
 	ui32RegVal = (1<<EUR_CR_MASTER_BIF_MMU_CTRL_ADDR_HASH_MODE_SHIFT);
 	#else
 	ui32RegVal = (1<<EUR_CR_MASTER_BIF_MMU_CTRL_ADDR_HASH_MODE_SHIFT) | EUR_CR_MASTER_BIF_MMU_CTRL_PREFETCHING_ON_MASK; 
 	#endif
 	#if !defined(FIX_HW_BRN_31620) && !defined(FIX_HW_BRN_31671)
-	
+	/* enable the DC TLB */
 	ui32RegVal |= EUR_CR_MASTER_BIF_MMU_CTRL_ENABLE_DC_TLB_MASK;
 	#endif
 
-	
+	/* Master bank */
 	OSWriteHWReg(psDevInfo->pvRegsBaseKM, EUR_CR_MASTER_BIF_MMU_CTRL, ui32RegVal);
 	PDUMPREGWITHFLAGS(SGX_PDUMPREG_NAME, EUR_CR_MASTER_BIF_MMU_CTRL, ui32RegVal, ui32PDUMPFlags);
 
 	#if defined(FIX_HW_BRN_31278) || defined(FIX_HW_BRN_32085)
-	
+	/* disable prefetch */
 	ui32RegVal = (1<<EUR_CR_BIF_MMU_CTRL_ADDR_HASH_MODE_SHIFT);
 	#else
 	ui32RegVal = (1<<EUR_CR_BIF_MMU_CTRL_ADDR_HASH_MODE_SHIFT) | EUR_CR_BIF_MMU_CTRL_PREFETCHING_ON_MASK; 
 	#endif
 	#if !defined(FIX_HW_BRN_31620) && !defined(FIX_HW_BRN_31671)
-	
+	/* enable the DC TLB */
 	ui32RegVal |= EUR_CR_BIF_MMU_CTRL_ENABLE_DC_TLB_MASK;
 	#endif
 
-	
+	/* Per-core */
 	{
 		IMG_UINT32 ui32Core;
 
@@ -661,6 +800,9 @@ IMG_VOID SGXReset(PVRSRV_SGXDEV_INFO	*psDevInfo,
 	
 	PDUMPCOMMENTWITHFLAGS(ui32PDUMPFlags, "End of SGX MP reset sequence\r\n");
 }	
-#endif 
+#endif /* SGX_FEATURE_MP */
 
 
+/******************************************************************************
+ End of file (sgxreset.c)
+******************************************************************************/
